@@ -13,8 +13,7 @@ use IO::Compress::Gzip qw($GzipError);
 use IO::Uncompress::Gunzip qw($GunzipError);
 
 #use Data::Dumper;
-
-#use Devel::Size           qw(size total_size);
+use Devel::Size qw(total_size);
 use Convert::Pheno;
 use Convert::Pheno::IO::FileIO;
 use Convert::Pheno::OMOP;
@@ -22,7 +21,7 @@ use Convert::Pheno::Schema;
 use Convert::Pheno::Mapping;
 use Exporter 'import';
 our @EXPORT =
-  qw(read_csv read_csv_stream read_redcap_dict_file read_mapping_file transpose_ohdsi_dictionary read_sqldump_stream read_sqldump sqldump2csv transpose_omop_data_structure write_csv open_filehandle load_exposures transpose_visit_occurrence get_headers);
+  qw(read_csv read_csv_stream read_redcap_dict_file read_mapping_file read_sqldump_stream read_sqldump sqldump2csv transpose_omop_data_structure write_csv open_filehandle load_exposures get_headers convert_table_aoh_to_hoh);
 
 use constant DEVEL_MODE => 0;
 
@@ -114,57 +113,6 @@ sub read_mapping_file {
     return $data_mapping_file;
 }
 
-sub transpose_ohdsi_dictionary {
-
-    my $data   = shift;
-    my $column = 'concept_id';
-
-    # The idea is the following:
-    # $data comes as an array (from SQL/CSV)
-    #
-    # $VAR1 = [
-    #          {
-    #            'concept_class_id' => '4-char billing code',
-    #            'concept_code' => 'K92.2',
-    #            'concept_id' => 35208414,
-    #            'concept_name' => 'Gastrointestinal hemorrhage, unspecified',
-    #            'domain_id' => 'Condition',
-    #            'invalid_reason' => undef,
-    #            'standard_concept' => undef,
-    #            'valid_end_date' => '2099-12-31',
-    #            'valid_start_date' => '2007-01-01',
-    #            'vocabulary_id' => 'ICD10CM'
-    #          },
-    #
-    # and we convert it to hash to allow for quick searches by 'concept_id'
-    #
-    # $VAR1 = {
-    #          '1107830' => {
-    #                         'concept_class_id' => 'Ingredient',
-    #                         'concept_code' => 28889,
-    #                         'concept_id' => 1107830,
-    #                         'concept_name' => 'Loratadine',
-    #                         'domain_id' => 'Drug',
-    #                         'invalid_reason' => undef,
-    #                         'standard_concept' => 'S',
-    #                         'valid_end_date' => '2099-12-31',
-    #                         'valid_start_date' => '1970-01-01',
-    #                         'vocabulary_id' => 'RxNorm'
-    #                         },
-    #
-    # NB: We store all columns yet we'll use 4:
-    # 'concept_id', 'concept_code', 'concept_name', 'vocabulary_id'
-    # Note that we're duplicating @$data with $hoh
-    #my $hoh = { map { $_->{$column} => $_ } @{$data} }; <--map is slower than for
-    my $hoh;
-    for my $item ( @{$data} ) {
-        $hoh->{ $item->{$column} } = $item;
-    }
-
-    #say "transpose_ohdsi_dictionary:", to_gb( total_size($hoh) ) if DEVEL_MODE;
-    return $hoh;
-}
-
 sub read_sqldump_stream {
 
     my $arg     = shift;
@@ -177,6 +125,9 @@ sub read_sqldump_stream {
     my $table_name    = $self->{omop_tables}[0];
     my $table_name_lc = lc($table_name);
 
+    # Define variables that modify what we load
+    my $max_lines_sql = $self->{max_lines_sql};
+
     # Open filehandles
     my $fh_in  = open_filehandle( $filein,  'r' );
     my $fh_out = open_filehandle( $fileout, 'a' );
@@ -219,7 +170,7 @@ sub read_sqldump_stream {
             # Order matters. We exit before loading
             last if $line =~ /^\\\.$/;
 
-            # Solitting by tab, it's ok
+            # Splitting by tab, it's ok
             my @fields = split /\t/, $line;
 
             # Using tmp hashref to load all fields at once with slice
@@ -244,12 +195,15 @@ sub read_sqldump_stream {
             # Only after encoding we are able to discard 'null'
             say $fh_out $encoded_data if $encoded_data ne 'null';
 
+            # adhoc filter to speed-up development
+            last if $count == $max_lines_sql;
+
             # Print if verbose
             say "Rows processed: $count"
               if ( $self->{verbose} && $count % 10_000 == 0 );
         }
     }
-    say "==============\nRows total:     $count\n" if $self->{verbose};
+    say "==============\nRows processed(total): $count\n" if $self->{verbose};
 
     #say $fh_out "]"; # not needed
 
@@ -264,7 +218,7 @@ sub encode_omop_stream {
     my ( $table_name, $hash_slice, $person, $count, $self ) = @_;
 
     # *** IMPORTANT ***
-    # We only print person_id ONCE!!!
+    # Table PERSON only has 1 individual
     my $person_id = $hash_slice->{person_id};
     my $data      = {
         $table_name => [$hash_slice],
@@ -310,6 +264,9 @@ sub read_sqldump {
     # ......
     # \.
 
+    # Verbose 
+    say "Reading the SQL dump...\n" if $self->{verbose};
+
     # Start reading the SQL dump
     my $fh = open_filehandle( $filepath, 'r' );
 
@@ -339,7 +296,8 @@ sub read_sqldump {
         next unless any { $_ eq $table_name } @omop_tables;
 
         # Say if verbose
-        say "Processing table ... <$table_name>" if $self->{verbose};
+        say "Loading <$table_name> in memory..."
+          if $self->{verbose};
 
         # Discarding first line
         shift @lines;
@@ -387,17 +345,20 @@ sub read_sqldump {
 
             # adhoc filter to speed-up development
             last if $count == $max_lines_sql;
-            say "Rows processed: $count"
+            say "Rows read: $count"
               if ( $self->{verbose} && $count % 1_000 == 0 );
 
         }
 
         # Print if verbose
-        say "==============\nRows total:     $count\n" if $self->{verbose};
+        say "==============\nRows read(total): $count\n" if $self->{verbose};
     }
     close $fh;
 
-    #say total_size($data) and die;
+    # RAM Usage
+    say ram_usage_str( 'read_sqldump', $data )
+      if ( DEVEL_MODE || $self->{verbose} );
+
     return $data;
 }
 
@@ -433,7 +394,7 @@ sub sqldump2csv {
 
 sub transpose_omop_data_structure {
 
-    my $data = shift;
+    my ( $self, $data ) = @_;
 
     # The situation is the following, $data comes in format:
     #
@@ -488,13 +449,15 @@ sub transpose_omop_data_structure {
     #                   }
     #        };
 
+    # Debug messages
+    say "> Transposing OMOP data..." if $self->{debug};
     my $omop_person_id = {};
 
     # Only performed for $omop_main_table
     for my $table ( @{ $omop_main_table->{$omop_version} } ) {    # global
 
-        # Loop over tables
-        for my $item ( @{ $data->{$table} } ) {
+        # We void the table when reading to avoid data duplication in RAM
+        while ( my $item = shift @{ $data->{$table} } ) {         # We want to keep order (!pop)
 
             if ( exists $item->{person_id} && $item->{person_id} ) {
                 my $person_id = $item->{person_id};
@@ -512,7 +475,7 @@ sub transpose_omop_data_structure {
         }
     }
 
-    # To get back unused memory for later..
+    # To get any unused memory back to Perl
     $data = undef;
 
     # Finally we get rid of the 'person_id' key and return values as an array
@@ -541,53 +504,21 @@ sub transpose_omop_data_structure {
     #          }
     #        ];
     # NB: We nsort keys to always have the same result but it's not needed
-    # v1 - Easier but duplicates data structure
-    # my $aoh = [ map { $omop_person_id->{$_} } nsort keys %{$omop_person_id} ];
-    # v2 - This version cleans memory after loading $aoh  <=== Implemented
+    say "> Sorting OMOP data by <person_id>..." if $self->{debug};
+
     my $aoh;
     for my $key ( nsort keys %{$omop_person_id} ) {
         push @{$aoh}, $omop_person_id->{$key};
-        delete $omop_person_id->{$key};
+        delete $omop_person_id->{$key};    # To avoid data duplication
     }
-    if (DEVEL_MODE) {
 
-        #say 'transpose_omop_data_structure(omop_person_id):',
-        #  to_gb( total_size($omop_person_id) );
-        #say 'transpose_omop_data_structure(map):', to_gb( total_size($aoh) );
-    }
+    # RAM usage
+    say ram_usage_str( 'transpose_omop_data_structure', $aoh )
+      if ( DEVEL_MODE || $self->{verbose} );
+
     return $aoh;
 }
 
-sub transpose_visit_occurrence {
-
-    my $data = shift;    # arrayref
-
-    # Going from
-    #$VAR1 = [
-    #        {
-    #          'admitting_source_concept_id' => 0,
-    #          'visit_occurrence_id' => 85,
-    #          ...
-    #        }
-    #      ];
-
-    # To
-    #$VAR1 = {
-    #        '85' => {
-    #                  'admitting_source_concept_id' => 0,
-    #                  'visit_occurrence_id' => 85,
-    #                  ...
-    #                }
-    #      };
-    #my $hash = { map { $_->{visit_occurrence_id} => $_ } @$data }; # map is slower than for
-    my $hash;
-    for my $item (@$data) {
-        my $key = $item->{visit_occurrence_id};    # otherwise $item->{visit_occurrence_id} goes from Int to Str in JSON and tests fail
-        $hash->{$key} = $item;
-    }
-    return $hash;
-}
-
 sub read_csv {
 
     my $arg      = shift;
@@ -794,7 +725,13 @@ sub to_gb {
 
     # base 2 => 1,073,741,824
     my $gb = $bytes / 1_073_741_824;
-    return sprintf( '%8.4f', $gb ) . ' GB';
+    return sprintf( '%9.4f', $gb ) . ' GB';
+}
+
+sub ram_usage_str {
+
+    my ( $func, $data ) = @_;
+    return qq/***RAM Usage***($func):/ . to_gb( total_size($data) ) . "\n";
 }
 
 sub load_exposures {
@@ -865,4 +802,94 @@ sub array_ref_to_hash {
     return \%hash;
 }
 
+sub convert_table_aoh_to_hoh {
+
+    my ( $data, $table, $self ) = @_;
+
+    my %table_cursor =
+      map { $_ => $data->{$_} } qw(CONCEPT PERSON VISIT_OCCURRENCE);
+    my %table_id = (
+        CONCEPT          => 'concept_id',
+        PERSON           => 'person_id',
+        VISIT_OCCURRENCE => 'visit_occurrence_id'
+    );
+    my $array_ref = $table_cursor{$table};
+    my $id        = $table_id{$table};
+
+    ###########
+    # CONCEPT #
+    ###########
+
+    # $VAR1 = [
+    #          {
+    #            'concept_class_id' => '4-char billing code',
+    #            'concept_code' => 'K92.2',
+    #            'concept_id' => 35208414,
+    #            'concept_name' => 'Gastrointestinal hemorrhage, unspecified',
+    #            'domain_id' => 'Condition',
+    #            'invalid_reason' => undef,
+    #            'standard_concept' => undef,
+    #            'valid_end_date' => '2099-12-31',
+    #            'valid_start_date' => '2007-01-01',
+    #            'vocabulary_id' => 'ICD10CM'
+    #          },
+    #
+    # and we convert it to hash to allow for quick searches by 'concept_id'
+    #
+    # $VAR1 = {
+    #          '1107830' => {
+    #                         'concept_class_id' => 'Ingredient',
+    #                         'concept_code' => 28889,
+    #                         'concept_id' => 1107830,
+    #                         'concept_name' => 'Loratadine',
+    #                         'domain_id' => 'Drug',
+    #                         'invalid_reason' => undef,
+    #                         'standard_concept' => 'S',
+    #                         'valid_end_date' => '2099-12-31',
+    #                         'valid_start_date' => '1970-01-01',
+    #                         'vocabulary_id' => 'RxNorm'
+    #                         },
+    #
+    # NB: We store all columns yet we'll use 4:
+    # 'concept_id', 'concept_code', 'concept_name', 'vocabulary_id'
+
+    ####################
+    # VISIT_OCCURRENCE #
+    ####################
+
+    # Going from
+    #$VAR1 = [
+    #        {
+    #          'admitting_source_concept_id' => 0,
+    #          'visit_occurrence_id' => 85,
+    #          ...
+    #        }
+    #      ];
+
+    # To
+    #$VAR1 = {
+    #        '85' => {
+    #                  'admitting_source_concept_id' => 0,
+    #                  'visit_occurrence_id' => 85,
+    #                  ...
+    #                }
+    #      };
+
+    # Initialize the new hash for transformed data
+    my $hoh = {};
+
+    # Iterate over the array and build the hash while clearing the array
+    while ( my $item = pop @{$array_ref} ) {    #faster than shift (order irrelevant here)
+        my $key = $item->{$id};                 # avoid stringfication
+        $hoh->{$key} = $item;
+    }
+
+    # The original array @{ $self->{data}{$table} is now empty
+    # RAM Usage
+    say ram_usage_str( "convert_table_aoh_to_hoh($table)", $hoh )
+      if ( $self->{verbose} || DEVEL_MODE );
+
+    return $hoh;
+}
+
 1;
